************************************************************************
* NEXTDATE() -- a UDF by Jeff Riedl  (ATBBS "Student")
*
*  Date: 5/6/1991
*   For: dBASE IV v 1.1
*  Note: Handy for Billing Cycles, Appointments, Payables etc!
* Usage: NEXTDATE({date},string)
*        where {date} is a date expression (starting date)
*        and the string is one of the Interval Codes listed below
*        or a number of days (in Character data type).
*
* Thanks to Todd W. Lindley  (ATBBS "Slice") for the code in DayCheck!
* Check out Tcal.ZIP in the dBASE4 Library for a neat Popup Calendar .PRG
*
* Your acknowledgement in any public domain / shareware that utilizes this
* UDF, or concepts contained herein, is appreciated.
*
************************************************************************

* NextDate.PRG is a little demo of how NEXTDATE() works!

CLEAR
SET TALK OFF
@  1,30 TO 3,48 DOUBLE COLOR gr+/bg 
@  2,31 SAY " NEXTDATE() DEMO " COLOR w+/n 
@  2,49 SAY "  " COLOR w+/n 
@  3,49 SAY "  " COLOR w+/n 
@  4,32 SAY "                   " COLOR w+/n 
@ 16, 2 SAY "Welcome to NextDate, a demo of the NextDate() UDF by Jeff Riedl!"
@ 17, 2 SAY "Enter a date, an interval code or number of days, and NextDate returns"
@ 18, 2 SAY "the results.  This is a great little UDF for billing, payables, &"
@ 19, 2 SAY "appointments.  And yes, it does handle Leap Year!"

DO WHILE LASTKEY() # 27
	m->dt = {}
	m->in = "  "
	@  7, 7 SAY "Enter a date:" GET m->dt ;
		MESSAGE "Pick a date, any date"
	read
	@  9, 2 SAY "Enter an interval:" GET m->in ;
		MESSAGE "Valid interval Codes: D, W, BW, SM, M, BM, Q, SA, A, or number of days."
	read
	@ 11, 9 SAY "Next Date ="
	@ 11,21 SAY NEXTDATE(m->dt,m->in)
	@ 14,13 SAY "Press 'Esc' to Exit Demo / Any other key to continue." COLOR gr+/b
	@ 24, 0
	set curs off
	x=inkey(0)
	@  7, 0 CLEAR
	set curs on
ENDDO
rele m->dt, m->in

*  Valid Interval Codes include "D","W","BW","SM","M","BM","Q","SA","A", or a 
*                               number of days from the start date.
*
*  The Function can be used in an @ SAY/GET along with a cycle in a Client.dbf, or
*  with a REPLACE command - I've found it especially handy for Billing programs
*  where clients are on different billing cycles or payment plans!
*
*  NOTE:  This UDF handles Leap Year & 'pushes' non-existent dates to the end of
*	  the previous month (i.e. a calculation for 2/30/91 would return 2/28/91).
*

*--------------------------This is the UDF to include in your Procedur.PRG FILE:

FUNCTION NextDate			&& Figures the next date based on given Date & Interval
PARAMETERS ld_olddate, lc_intervl	&& Date to work from & Character String defining the interval

*--Clean up the interval
lc_intervl = UPPER(LTRIM(TRIM(lc_intervl)))	

PRIVATE ALL
STORE 0 TO ln_newmo,ln_newdy,ln_newyr,ln_olddy
STORE {} TO ld_newdate

DO CASE								&& Valid Interval Codes Include:

	CASE lc_intervl = "D"					&& "D" - Daily (Every Day)
		ld_newdate = ld_olddate + 1
		
	CASE lc_intervl = "W"					&& "W" - Weekly (Every Week)
		ld_newdate = ld_olddate + 7
		
	CASE lc_intervl = "BW"					&& "BW" - BiWeekly (Every Other Week)
		ld_newdate = ld_olddate + 14
		
	CASE lc_intervl = "SM"					&& "SM" - SemiMonthly (Twice Monthly on the same 'Days')
		ln_olddy = DAY(ld_olddate)
		IF ln_olddy <= 14
			ld_newdate = ld_olddate + 14
		ELSE 
			ln_newdy = ln_olddy - 14
			ln_newmo = MONTH(ld_olddate)+1
			DO MonthCheck
			ld_newdate = CTOD(LTRIM(STR(ln_newmo))+"/"+LTRIM(STR(ln_newdy))+"/"+LTRIM(STR(ln_newyr)))
		ENDIF
		
	CASE lc_intervl = "M"					&& "M" - Monthly (Same day of the month) 
		ln_newmo = MONTH(ld_olddate)+1
		DO MonthCheck
		ln_newdy = DAY(ld_olddate)
		DO DayCheck
		ld_newdate = CTOD(LTRIM(STR(ln_newmo))+"/"+LTRIM(STR(ln_newdy))+"/"+LTRIM(STR(ln_newyr)))
		
	CASE lc_intervl = "BM"					&& "BM" - BiMonthly (Every other month)
		ln_newmo = MONTH(ld_olddate)+2
		DO MonthCheck
		ln_newdy = DAY(ld_olddate)
		DO DayCheck
		ld_newdate = CTOD(LTRIM(STR(ln_newmo))+"/"+LTRIM(STR(ln_newdy))+"/"+LTRIM(STR(ln_newyr)))
		
	CASE lc_intervl = "Q"					&& "Q" - Quarterly
		ln_newmo = MONTH(ld_olddate)+3
		DO MonthCheck
		ln_newdy = DAY(ld_olddate)
		DO DayCheck
		ld_newdate = CTOD(LTRIM(STR(ln_newmo))+"/"+LTRIM(STR(ln_newdy))+"/"+LTRIM(STR(ln_newyr)))
		
	CASE lc_intervl = "SA"					&& "SA" - SemiAnnually (Twice Yearly)
		ln_newmo = MONTH(ld_olddate)+6
		DO MonthCheck
		ln_newdy = DAY(ld_olddate)
		DO DayCheck
		ld_newdate = CTOD(LTRIM(STR(ln_newmo))+"/"+LTRIM(STR(ln_newdy))+"/"+LTRIM(STR(ln_newyr)))

	CASE lc_intervl = "A"					&& "A" - Annually (Same Day each year)
		ln_newmo = MONTH(ld_olddate)
		ln_newdy = DAY(ld_olddate)
		ln_newyr = YEAR(ld_olddate)+1
		ld_newdate = CTOD(LTRIM(STR(ln_newmo))+"/"+LTRIM(STR(ln_newdy))+"/"+LTRIM(STR(ln_newyr)))

	OTHERWISE						&& Interval specified by a number of days.
		ld_newdate = ld_olddate + VAL(lc_intervl)
ENDCASE


RETURN ld_newdate
*-- EOP: NextDate


PROCEDURE MonthCheck			&& Corrects Months Higher than 12 & Sets Correct Year
IF ln_newmo > 12
	ln_newmo = ln_newmo - 12
	ln_newyr = YEAR(ld_olddate)+1
ELSE
	ln_newyr = YEAR(ld_olddate)
ENDIF
*-- EOP: MonthCheck


PROCEDURE DayCheck			&& Corrects Days Higher than the number of days in the month
ld_beg=CTOD(STR(ln_newmo,2,0)+"/"+"01/"+STR(ln_newyr,4,0))		&& CALCULATE 1ST OF MONTH AND BEGINNING OF NEXT MONTH
ld_next=CTOD(STR(ln_newmo+1,2,0)+"/"+"01/"+STR(ln_newyr,4,0))
ln_lastday=ld_next-ld_beg						&& CALCULATE # OF DAYS IN MONTH
IF ln_newdy > ln_lastday
	ln_newdy = ln_lastday
ENDIF
*-- EOP: DayCheck

********************** END OF UDF & ASSOCIATED PROCEDURES **********************
